/*----------------------------------------------------------------------------
 * Name:    Uart1
 * Purpose: USART usage for STM32
 * Version: V1.00
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * Copyright (c) 2005-2007 Keil Software. All rights reserved.
 *----------------------------------------------------------------------------*/
#include "stm32f10x.h"
#include <stdio.h>

/*----------------------------------------------------------------------------
  Notes:
  The length of the receive and transmit buffers must be a power of 2.
  Each buffer has a next_in and a next_out index.
  If next_in = next_out, the buffer is empty.
  (next_in - next_out) % buffer_size = the number of characters in the buffer.
 *----------------------------------------------------------------------------*/
#define TBUF_SIZE   1024     /*** Must be a power of 2 (2,4,8,16,32,64,128,256,512,...) ***/
#define RBUF_SIZE   256      /*** Must be a power of 2 (2,4,8,16,32,64,128,256,512,...) ***/

/*----------------------------------------------------------------------------
   Check TBUF_SIZE and RBUF_SIZE
 *----------------------------------------------------------------------------*/
#if TBUF_SIZE < 2
#error TBUF_SIZE is too small.  It must be larger than 1.
#elif ((TBUF_SIZE & (TBUF_SIZE-1)) != 0)
#error TBUF_SIZE must be a power of 2.
#endif

#if RBUF_SIZE < 2
#error RBUF_SIZE is too small.  It must be larger than 1.
#elif ((RBUF_SIZE & (RBUF_SIZE-1)) != 0)
#error RBUF_SIZE must be a power of 2.
#endif

/*----------------------------------------------------------------------------
   Define buffer structure, Tx/Rx buffers and buffer length calculating macros
 *----------------------------------------------------------------------------*/
struct buf_st {
  unsigned int in;                           // Next In Index
  unsigned int out;                          // Next Out Index
  char buf [TBUF_SIZE];                      // Buffer
};

static struct buf_st rbuf = { 0, 0, };
#define SIO_RBUFLEN ((unsigned short)(rbuf.in - rbuf.out))

static struct buf_st tbuf = { 0, 0, };
#define SIO_TBUFLEN ((unsigned short)(tbuf.in - tbuf.out))

static unsigned int tx_restart = 1;          // NZ if TX restart is required

/*----------------------------------------------------------------------------
  USART1_IRQHandler
  Handles USART1 global interrupt request.
 *----------------------------------------------------------------------------*/
void USART1_IRQHandler (void) {
  volatile unsigned int IIR;
  struct buf_st *p;
  IIR = USART1->SR;                          // copy status register
  if (IIR & USART_SR_RXNE) {                 // read interrupt
    USART1->SR &= ~USART_SR_RXNE;	         // clear interrupt
    p = &rbuf;
    if (((p->in - p->out) & ~(RBUF_SIZE-1)) == 0) {
      p->buf [p->in & (RBUF_SIZE-1)] = (USART1->DR & 0x1FF);
      p->in++;
    }
  }
  if (IIR & USART_SR_TXE) {
    USART1->SR &= ~USART_SR_TXE;             // clear interrupt
    p = &tbuf;
    if (p->in != p->out) {
      USART1->DR = (p->buf [p->out & (TBUF_SIZE-1)] & 0x1FF);
      p->out++;
      tx_restart = 0;
    }
    else {
      tx_restart = 1;
      USART1->CR1 &= ~USART_SR_TXE;	         // disable TX interrupt if nothing to send
    }
  }
}

/*------------------------------------------------------------------------------
  buffer_Init
  initialize the buffers
 *------------------------------------------------------------------------------*/
void buffer_Init (void) {
  tbuf.in = 0;                               // Clear com buffer indexes
  tbuf.out = 0;
  tx_restart = 1;
  rbuf.in = 0;
  rbuf.out = 0;
}

/*------------------------------------------------------------------------------
  SendChar - transmit a character
 *------------------------------------------------------------------------------*/
int SendChar (int c) {
  struct buf_st *p = &tbuf;
  if (SIO_TBUFLEN >= TBUF_SIZE)
    return (-1);                             // If buffer is full, return an error code
  p->buf [p->in & (TBUF_SIZE - 1)] = c;      // Add data to the transmit buffer.
  p->in++;
  if (tx_restart) {                          // If transmit interrupt is disabled, enable it
    tx_restart = 0;
    USART1->CR1 |= USART_SR_TXE;             // enable TX interrupt
  }
  return (0);
}

/*------------------------------------------------------------------------------
  GetKey - receive a character
 *------------------------------------------------------------------------------*/
int GetKey (void) {
  struct buf_st *p = &rbuf;
  if (SIO_RBUFLEN == 0)
    return (-1);                             // If buffer is empty, return an error code
  return (p->buf[(p->out++) & (RBUF_SIZE-1)]);
}

/*----------------------------------------------------------------------------
  USART1 s a PB6/PB7 kivezetsek inicializlsa
 *----------------------------------------------------------------------------*/
void USART1_init (void) {
  RCC->APB2ENR |= RCC_APB2ENR_USART1EN;      // USART1 engedlyezse
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;        // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;        // GPIOB rajel engedlyezs
  AFIO->MAPR |= AFIO_MAPR_USART1_REMAP;      // Belltja USART1 REMAP bitjt 
  //--- PB6 konfigurlsa USART1 Tx digitlis kimenetknt -----------
  GPIOB->CRL &= ~(GPIO_CRL_MODE6|GPIO_CRL_CNF6); // PB6 CNF s Mode bitek trlse
  GPIOB->CRL |=  GPIO_CRL_MODE6_0 |          // PA2 pushpull alternate kimenet, max. 10 MHz
                 GPIO_CRL_CNF6_1;            // CNF: 10  MODE: 01
  //--- PB7 konfigurlsa digitlis bemenetknt  -----------
  GPIOB->CRL &= ~(GPIO_CRL_MODE7|GPIO_CRL_CNF7); // PB7 CNF s Mode bitek trlse
  GPIOB->CRL |=  GPIO_CRL_CNF7_0;            // PB7 digitlis bemenet CNF: 01  MODE: 00	
  //--- USART1 konfigurlsa: Tx&Rx, 9600  8 N 1 mdba --------------
  USART1->BRR = 7500;                        // 9600 baud @ 72 MHz (APB2!)
  USART1->CR1 = USART_CR1_TE | USART_CR1_RE| // Tx, Rx engedlyezs, 8-bit adat
                USART_CR1_TXEIE |            // TXE  megszakts engedlyezse
                USART_CR1_RXNEIE;            // RXNE megszakts engedlyezse 
  NVIC_EnableIRQ(USART1_IRQn);               // USART1 megszakts engedlyezse
  USART1->CR2 = 0x0000;                      // 1 stop bit
  USART1->CR3 = 0x0000;                      // nincs adatfolyam-vezrls
  USART1->CR1 |= USART_CR1_UE;               // USART1 engedlyezs
}
